import numpy as np


def find_velocity_values_within_limit(skeleton_velocity_data, velocity_limit):
    """
    This function takes in a skeleton velocity data array and a limit and returns the indices of the values that are within the limit
    """
    indices = []
    for i in range(len(skeleton_velocity_data)):
        if abs(skeleton_velocity_data[i]) <= velocity_limit:
            indices.append(i+1) #add 1 to account for the difference in indices between the position and velocity data
    return indices

def find_matching_indices_in_lists(list_1,list_2,list_3,list_4):
    """
    This function takes in four lists and returns the indices of the values that are in all four lists
    """
    matching_values = [x for x in list_1 if x in list_2 and x in list_3 and x in list_4]

    return matching_values
def find_best_velocity_guess(skeleton_velocity_data, skeleton_indices, velocity_guess, iteration_range):
    """
    This function iterates over velocity data and tries to pare down to a single frame that has the closest velocity to 0 for all foot markers
    """

    right_heel_index = skeleton_indices.index('right_heel')
    right_toe_index = skeleton_indices.index('right_foot_index')
    left_heel_index = skeleton_indices.index('left_heel')
    left_toe_index = skeleton_indices.index('left_foot_index')

    skeleton_data_velocity_x_right_heel = skeleton_velocity_data[:,right_heel_index,0]
    skeleton_data_velocity_x_right_toe = skeleton_velocity_data[:,right_toe_index,0]
    skeleton_data_velocity_x_left_heel = skeleton_velocity_data[:,left_heel_index,0]
    skeleton_data_velocity_x_left_toe = skeleton_velocity_data[:,left_toe_index,0]

    #get a list of the frames where the velocity for that marker is within the velocity limit 
    right_heel_x_velocity_limits = find_velocity_values_within_limit(skeleton_data_velocity_x_right_heel, velocity_guess)
    right_toe_x_velocity_limits = find_velocity_values_within_limit(skeleton_data_velocity_x_right_toe, velocity_guess)
    left_heel_x_velocity_limits = find_velocity_values_within_limit(skeleton_data_velocity_x_left_heel, velocity_guess)
    left_toe_x_velocity_limits = find_velocity_values_within_limit(skeleton_data_velocity_x_left_toe, velocity_guess)

    #return a list of matching frame indices from the four lists generated above 
    matching_values = find_matching_indices_in_lists(right_heel_x_velocity_limits, right_toe_x_velocity_limits, left_heel_x_velocity_limits, left_toe_x_velocity_limits)
    matching_values = [x for x in matching_values if x>75]
    
    #print(matching_values)
    if len(matching_values) > 1 and velocity_guess > 0:
        #if there are multiple matching values, decrease the guess a little bit and run the function again
        #  
        velocity_guess = velocity_guess - iteration_range
        print('Current Velocity Guess:',velocity_guess, '| Number of Possible Frames:', len(matching_values), '| Possible Frames:', matching_values)
        matching_values, velocity_guess = find_best_velocity_guess(skeleton_velocity_data, skeleton_indices, velocity_guess, iteration_range)

        f = 2
    elif len(matching_values) == 0:
        #if there are no matching values (we decreased our guess too far), reset the guess to be a bit smaller and run the function again with smaller intervals between the guesses
        iteration_range = iteration_range/2
        matching_values, velocity_guess = find_best_velocity_guess(skeleton_velocity_data, skeleton_indices, velocity_guess + iteration_range*2, iteration_range)

        f = 2
    elif len(matching_values) == 1:
        print('Good Frame:', matching_values,'| Final Velocity Guess:',velocity_guess)

    return matching_values, velocity_guess


def find_good_frame(skeleton_data, skeleton_indices:list, initial_velocity_guess:float, debug = False):
    """
    Finds a frame (called the good frame) where the velocity of both feet are closest to 0

    Input: 
        skeleton data: a 3D numpy array of skeleton data in freemocap format
        skeleton indices: a list of joints being tracked by mediapipe/your 2d pose estimator
        initial velocity guess: just a starting guess for the optimizer. Can adjust if you're not getting the results you want
        debug: plots and displays the calculated good frame if True 
    """

    skeleton_velocity_data = np.diff(skeleton_data, axis=0)
    
    matching_values, velocity_guess = find_best_velocity_guess(skeleton_velocity_data, skeleton_indices, initial_velocity_guess, iteration_range=.1)

    good_frame = matching_values[0]

    return good_frame


from pyqtgraph.parametertree import Parameter,registerParameterType
from skellyforge.freemocap_utils.constants import(
    TASK_INTERPOLATION,
    TASK_FILTERING,
    TASK_FINDING_GOOD_FRAME,
    TASK_SKELETON_ROTATION,
    PARAM_METHOD,
    PARAM_ORDER,
    PARAM_CUTOFF_FREQUENCY,
    PARAM_SAMPLING_RATE,
    PARAM_ROTATE_DATA,
    PARAM_AUTO_FIND_GOOD_FRAME,
    PARAM_GOOD_FRAME
)


interpolation_settings = [
    {"name": TASK_INTERPOLATION.title(), "type": "group", "children": [
        {"name": PARAM_METHOD, "type": "list", "values": ["linear", "cubic", "spline"]},
        {"name": "Order (only used in spline interpolation)", "type": "int", "value":3, "step":1}
    ]}
]

filter_settings = [
        {"name": TASK_FILTERING.title(), "type": "group", "children": [
        {"name": "Filter Type", "type": "list", "values": ["Butterworth Low Pass"]},
        {"name": PARAM_ORDER, "type":"int","value":4, "step":.1},
        {"name": PARAM_CUTOFF_FREQUENCY, "type": "float", "value": 6.0, "step": 0.1},
        {"name": PARAM_SAMPLING_RATE, "type": "float", "value": 30.0, "step": 0.1},
    ]}
]

rotation_settings = [
    {"name": TASK_SKELETON_ROTATION.title(), "type": "group", "children": [
        {"name": PARAM_ROTATE_DATA, "type": "bool", "value": True},
        {"name": "Instructions", "type": "str", "value": "Uncheck 'Auto-find Good Frame' to type in the good frame manually.", "readonly": True},
        {"name": PARAM_AUTO_FIND_GOOD_FRAME, "type": "bool", "value": True},
        {"name": PARAM_GOOD_FRAME, "type": "str", "value": "", "step": 1},
    ]}
]

class CustomRotationParam(Parameter):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.rotate_data_param = self.child(TASK_SKELETON_ROTATION.title()).child(PARAM_ROTATE_DATA)
        self.auto_find_good_frame_param = self.child(TASK_SKELETON_ROTATION.title()).child(PARAM_AUTO_FIND_GOOD_FRAME)
        self.good_frame_param = self.child(TASK_SKELETON_ROTATION.title()).child(PARAM_GOOD_FRAME)

        self.rotate_data_param.sigValueChanged.connect(self.rotate_data_changed)
        self.auto_find_good_frame_param.sigValueChanged.connect(self.auto_find_good_frame_changed)

    def rotate_data_changed(self, value):
        enabled = value.value()
        self.auto_find_good_frame_param.setOpts(enabled=enabled)
        self.good_frame_param.setOpts(enabled=enabled)

    def auto_find_good_frame_changed(self, value):
        if value.value():
            self.good_frame_param.setValue("")
            self.good_frame_param.setOpts(readonly=True)
        else:
            if not self.good_frame_param.value():
                self.good_frame_param.setValue("0")
            self.good_frame_param.setOpts(readonly=False)


interpolation_params = Parameter.create(name='interp_params', type='group', children=interpolation_settings)
filter_params = Parameter.create(name='filter_params',type='group', children=filter_settings )

registerParameterType('CustomRotationParam',CustomRotationParam)
rotation_params = Parameter.create(name='rotation_params', type='CustomRotationParam', children=rotation_settings) #the 'type' here refers to the parameter type we made the line above



if __name__ == "__main__":

    def get_all_parameter_values(parameter_object):
        values = {}
        for child in parameter_object.children(): #using this just to access the second level of the parameter tree
            if child.hasChildren():
                for grandchild in child.children():
                    values[grandchild.name()] = grandchild.value()
            else:
                values[child.name()] = child.value()
        return values

    values = get_all_parameter_values(filter_params)
    f =2 


from rich.progress import track
from pathlib import Path

mediapipe_indices = ['nose',
    'left_eye_inner',
    'left_eye',
    'left_eye_outer',
    'right_eye_inner',
    'right_eye',
    'right_eye_outer',
    'left_ear',
    'right_ear',
    'mouth_left',
    'mouth_right',
    'left_shoulder',
    'right_shoulder',
    'left_elbow',
    'right_elbow',
    'left_wrist',
    'right_wrist',
    'left_pinky',
    'right_pinky',
    'left_index',
    'right_index',
    'left_thumb',
    'right_thumb',
    'left_hip',
    'right_hip',
    'left_knee',
    'right_knee',
    'left_ankle',
    'right_ankle',
    'left_heel',
    'right_heel',
    'left_foot_index',
    'right_foot_index']

mediapipe_connections = {'shoulders':['left_shoulder','right_shoulder'],
    'hips':['left_hip','right_hip'],
    'torso_left': ['left_shoulder','left_hip'],
    'torso_right':['right_shoulder','right_hip'],
    'left_upper_arm':['left_shoulder','left_elbow'],
    'left_lower_arm':['left_elbow','left_wrist'],
    'left_upper_leg':['left_hip','left_knee'],
    'left_lower_leg':['left_knee','left_ankle'],
    'left_heel_connection':['left_ankle','left_heel'],
    'left_foot':['left_heel','left_foot_index'],

    'right_upper_arm':['right_shoulder','right_elbow'],
    'right_lower_arm':['right_elbow','right_wrist'],
    'right_upper_leg':['right_hip','right_knee'],
    'right_lower_leg':['right_knee','right_ankle'],
    'right_heel_connection':['right_ankle','right_heel'],
    'right_foot':['right_heel','right_foot_index']}

reprojection_error_mediapipe_connections = {'left_upper_arm':['left_shoulder','left_elbow'],
    'left_lower_arm':['left_elbow','left_wrist'],
    'left_upper_leg':['left_hip','left_knee'],
    'left_lower_leg':['left_knee','left_ankle'],
    'left_foot':['left_ankle','left_heel','left_foot_index'],
    'right_upper_arm':['right_shoulder','right_elbow'],
    'right_lower_arm':['right_elbow','right_wrist'],
    'right_upper_leg':['right_hip','right_knee'],
    'right_lower_leg':['right_knee','right_ankle'],
    'right_foot':['right_ankle','right_heel','right_foot_index']}


def get_joint_coordinates_from_name(frame:int,joint_name:str,pose_estimation_markers,skel_3d_data):
    joint_index_number = pose_estimation_markers.index(joint_name)
    joint_coordinates = skel_3d_data[frame,joint_index_number,:]

    return joint_coordinates


def build_skeleton(skel_3d_data,pose_estimation_markers,pose_estimation_connections):

    num_frames = skel_3d_data.shape[0]

    skeleton_connection_coordinates = []

    for frame in track(range(num_frames), 'Building Mediapipe Skeleton'):
        this_frame_connection_dict = {}
        for connection in pose_estimation_connections:
            
            joint_1_name = pose_estimation_connections[connection][0]
            joint_2_name = pose_estimation_connections[connection][1]

            joint_1_coordinates = get_joint_coordinates_from_name(frame,joint_1_name,pose_estimation_markers,skel_3d_data)
            joint_2_coordinates = get_joint_coordinates_from_name(frame,joint_2_name,pose_estimation_markers,skel_3d_data)

            this_connection_coordinates = [joint_1_coordinates,joint_2_coordinates]

            this_frame_connection_dict[connection] = this_connection_coordinates
        skeleton_connection_coordinates.append(this_frame_connection_dict)

    return skeleton_connection_coordinates
    f = 2

def get_mediapipe_indices(joint_center:str):
    return mediapipe_indices.index(joint_center)



if __name__ == '__main__':
    freemocap_data_folder_path = Path(r'D:\freemocap2022\FreeMocap_Data')
    sessionID = 'sesh_2022-09-29_17_29_31'
    data_array_folder = 'DataArrays'
    array_name = 'mediaPipeSkel_3d.npy'


    mediapipe_indices.index('left_heel')
    f = 2


from PySide6.QtCore import Qt
from PySide6.QtWidgets import QSlider, QWidget,QLabel,QHBoxLayout

class FrameCountSlider(QWidget):
    def __init__(self, num_frames:int):
        super().__init__()

        self._layout = QHBoxLayout()
        self.setLayout(self._layout)

        self.slider = QSlider(Qt.Orientation.Horizontal)
        self.slider.setMaximum(0)
        
        self.label = QLabel(str(self.slider.value()))
        self.slider.valueChanged.connect(lambda: self.label.setText(str(self.slider.value())))

        self._layout.addWidget(self.label)
        self._layout.addWidget(self.slider)

        self.set_slider_range(num_frames)


    def set_slider_range(self,num_frames):
        self.slider_max = num_frames - 1
        self.slider.setValue(0)
        self.slider.setMaximum(self.slider_max)




from PySide6.QtWidgets import QWidget, QHBoxLayout
from skellyforge.freemocap_utils.postprocessing_widgets.visualization_widgets.skeleton_view_widget import SkeletonViewWidget


class SkeletonViewersContainer(QWidget):
    def __init__(self):
        super().__init__()

        layout = QHBoxLayout()

        self.raw_skeleton_viewer = SkeletonViewWidget('Raw data')
        layout.addWidget(self.raw_skeleton_viewer)

        self.processed_skeleton_viewer = SkeletonViewWidget('Post-processed data')
        layout.addWidget(self.processed_skeleton_viewer)

        self.setLayout(layout)

    def plot_raw_skeleton(self, raw_skeleton_data):
        self.raw_skeleton_viewer.load_skeleton(raw_skeleton_data)

    def plot_processed_skeleton(self, processed_skeleton_data):
        self.processed_skeleton_viewer.load_skeleton(processed_skeleton_data)

    def update_raw_viewer_plot(self, frame_number):
        self.raw_skeleton_viewer.replot(frame_number)

    def update_processed_viewer_plot(self, frame_number):
        if self.processed_skeleton_viewer.skeleton_loaded:
            self.processed_skeleton_viewer.replot(frame_number)



import numpy as np

from PySide6.QtWidgets import QWidget,QVBoxLayout, QPushButton, QGroupBox


from skellyforge.freemocap_utils.postprocessing_widgets.task_worker_thread import TaskWorkerThread
from skellyforge.freemocap_utils.postprocessing_widgets.parameter_tree_builder import create_filter_parameter_tree, create_filter_page_settings_dict
from skellyforge.freemocap_utils.postprocessing_widgets.visualization_widgets.timeseries_view_widget import TimeSeriesPlotterWidget
from skellyforge.freemocap_utils.postprocessing_widgets.visualization_widgets.marker_selector_widget import MarkerSelectorWidget
from skellyforge.freemocap_utils.postprocessing_widgets.stylesheet import groupbox_stylesheet

from skellyforge.freemocap_utils.constants import (
    TASK_INTERPOLATION,
    TASK_FILTERING,
)

class FilteringMenu(QWidget):
    def __init__(self, freemocap_raw_data:np.ndarray):
        super().__init__()

        self.setStyleSheet(groupbox_stylesheet)

        layout = QVBoxLayout()
        self.freemocap_raw_data = freemocap_raw_data
        self.processed_freemocap_data = None

        # Timeseries and marker selector groupbox
        self.time_series_groupbox = self.create_time_series_groupbox()
        self.filter_param_tree_groupbox = self.create_filtering_groupbox()

        layout.addWidget(self.time_series_groupbox)
        layout.addWidget(self.filter_param_tree_groupbox)

        self.run_filter_button = QPushButton('Run Filter')
        self.run_filter_button.clicked.connect(self.run_filter_task)
        layout.addWidget(self.run_filter_button)

        self.update_timeseries_plot()

        self.setLayout(layout)
        self.connect_signals_to_slots()

    def create_time_series_groupbox(self):
        groupbox = QGroupBox("View time series for a selected marker")
        time_series_layout = QVBoxLayout()
        self.marker_selector_widget = MarkerSelectorWidget()
        time_series_layout.addWidget(self.marker_selector_widget)
        self.time_series_plotter_widget = TimeSeriesPlotterWidget()
        time_series_layout.addWidget(self.time_series_plotter_widget)
        groupbox.setLayout(time_series_layout)
        return groupbox
    
    def create_filtering_groupbox(self):
        groupbox = QGroupBox("Filtering Parameters")
        filter_params_layout = QVBoxLayout()
        self.filter_param_tree = create_filter_parameter_tree()
        filter_params_layout.addWidget(self.filter_param_tree)
        groupbox.setLayout(filter_params_layout)
        return groupbox

    def update_timeseries_plot(self, reset_axes = True):
        self.time_series_plotter_widget.update_plot(marker_to_plot=self.marker_selector_widget.current_marker, original_freemocap_data=self.freemocap_raw_data , processed_freemocap_data=self.processed_freemocap_data,reset_axes = reset_axes)

    def connect_signals_to_slots(self):
        self.marker_selector_widget.marker_to_plot_updated_signal.connect(lambda: self.update_timeseries_plot(reset_axes=True))
    
    def run_filter_task(self):
        self.settings_dict = create_filter_page_settings_dict()
        self.worker_thread = TaskWorkerThread(
            raw_skeleton_data=self.freemocap_raw_data,
            task_list= [TASK_INTERPOLATION, 
                        TASK_FILTERING],
            settings=self.settings_dict,
            all_tasks_finished_callback=self.handle_filter_result)        
        self.worker_thread.start()


    def handle_filter_result(self, task_results: dict):
        self.processed_freemocap_data = task_results[TASK_FILTERING]['result']
        self.update_timeseries_plot(reset_axes=False)
       



from PySide6.QtCore import Signal
from PySide6.QtWidgets import QWidget,QVBoxLayout

import matplotlib
matplotlib.use('QtAgg')
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg
from matplotlib.figure import Figure

import numpy as np

from skellyforge.freemocap_utils.postprocessing_widgets.visualization_widgets.mediapipe_skeleton_builder import mediapipe_indices,mediapipe_connections,build_skeleton


class SkeletonViewWidget(QWidget):

    session_folder_loaded_signal = Signal()

    def __init__(self, plot_title:str):
        super().__init__()

        self._layout = QVBoxLayout()
        self.setLayout(self._layout)

        self.plot_title = plot_title
        self.fig,self.ax = self.initialize_skeleton_plot()
        self._layout.addWidget(self.fig)

        self.skeleton_loaded = False

        self.current_xlim = None
        self.current_ylim = None
        self.current_zlim = None


    def load_skeleton(self,skeleton_3d_data:np.ndarray):

        self.skeleton_3d_data = skeleton_3d_data
        self.mediapipe_skeleton = build_skeleton(self.skeleton_3d_data,mediapipe_indices,mediapipe_connections)
        self.reset_skeleton_3d_plot()

        self.skeleton_loaded = True

            
    def initialize_skeleton_plot(self):
        fig = Mpl3DPlotCanvas(self, width=5, height=4, dpi=100)
        ax = fig.figure.axes[0]

        ax.set_title(self.plot_title)
        return fig, ax

    def reset_skeleton_3d_plot(self):
        self.ax.cla()
        self.calculate_axes_means(self.skeleton_3d_data)
        self.skel_x,self.skel_y,self.skel_z = self.get_x_y_z_data(0)
        self.plot_skel(0,self.skel_x,self.skel_y,self.skel_z)


    def calculate_axes_means(self,skeleton_3d_data):
        self.mx_skel = np.nanmean(skeleton_3d_data[:,0:33,0])
        self.my_skel = np.nanmean(skeleton_3d_data[:,0:33,1])
        self.mz_skel = np.nanmean(skeleton_3d_data[:,0:33,2])
        self.skel_3d_range = 900

    def plot_skel(self,frame_number,skel_x,skel_y,skel_z):
        self.ax.scatter(skel_x,skel_y,skel_z)
        self.plot_skeleton_bones(frame_number)
        if self.current_xlim:
            self.ax.set_xlim([self.current_xlim[0],self.current_xlim[1]])
            self.ax.set_ylim([self.current_ylim[0],self.current_ylim[1]])
            self.ax.set_zlim([self.current_zlim[0],self.current_zlim[1]])
        else:
            self.ax.set_xlim([self.mx_skel-self.skel_3d_range, self.mx_skel+self.skel_3d_range])
            self.ax.set_ylim([self.my_skel-self.skel_3d_range, self.my_skel+self.skel_3d_range])
            self.ax.set_zlim([self.mz_skel-self.skel_3d_range, self.mz_skel+self.skel_3d_range])
        
        self.ax.set_title(self.plot_title)
        self.fig.figure.canvas.draw()

    def plot_skeleton_bones(self,frame_number):
            this_frame_skeleton_data = self.mediapipe_skeleton[frame_number]
            for connection in this_frame_skeleton_data.keys():
                line_start_point = this_frame_skeleton_data[connection][0] 
                line_end_point = this_frame_skeleton_data[connection][1]
                
                bone_x,bone_y,bone_z = [line_start_point[0],line_end_point[0]],[line_start_point[1],line_end_point[1]],[line_start_point[2],line_end_point[2]] 

                self.ax.plot(bone_x,bone_y,bone_z)

    def get_x_y_z_data(self, frame_number:int):
        skel_x = self.skeleton_3d_data[frame_number,:,0]
        skel_y = self.skeleton_3d_data[frame_number,:,1]
        skel_z = self.skeleton_3d_data[frame_number,:,2]

        return skel_x,skel_y,skel_z

    def replot(self, frame_number:int):
        skel_x,skel_y,skel_z = self.get_x_y_z_data(frame_number)
        self.current_xlim = self.ax.get_xlim()
        self.current_ylim = self.ax.get_ylim()
        self.current_zlim = self.ax.get_zlim()
        self.ax.cla()
        self.plot_skel(frame_number,skel_x,skel_y,skel_z)
        #self.label.setText(str(frame_number))


class Mpl3DPlotCanvas(FigureCanvasQTAgg):

    def __init__(self, parent=None, width=4, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111,projection = '3d')
        super(Mpl3DPlotCanvas, self).__init__(fig)







from skellyforge.freemocap_utils.constants import(
    TASK_INTERPOLATION,
    TASK_FILTERING,
    TASK_FINDING_GOOD_FRAME,
    TASK_SKELETON_ROTATION,
    PARAM_METHOD,
    PARAM_ORDER,
    PARAM_CUTOFF_FREQUENCY,
    PARAM_SAMPLING_RATE,
    PARAM_ROTATE_DATA,
    PARAM_AUTO_FIND_GOOD_FRAME,
    PARAM_GOOD_FRAME
)

default_settings = {
TASK_INTERPOLATION:
 {PARAM_METHOD: 'linear',
  PARAM_ORDER : 3
 },
 
 TASK_FILTERING: 
 {'Method' : 'Butterworth Low Pass Filter',
  PARAM_ORDER: 4,
  PARAM_CUTOFF_FREQUENCY: 6,
  PARAM_SAMPLING_RATE:30 
 },

 TASK_SKELETON_ROTATION:
 {PARAM_ROTATE_DATA: True,
  PARAM_AUTO_FIND_GOOD_FRAME: True,
  PARAM_GOOD_FRAME: None
 }
}




from pathlib import Path
import numpy as np

from PySide6.QtWidgets import QMainWindow, QApplication, QTabWidget, QWidget, QVBoxLayout

from skellyforge.freemocap_utils.postprocessing_widgets.menus.main_menu import MainMenu
from skellyforge.freemocap_utils.postprocessing_widgets.menus.interpolation_menu import InterpolationMenu
from skellyforge.freemocap_utils.postprocessing_widgets.menus.filtering_menu import FilteringMenu
import toml

class FileManager:
    def __init__(self, path_to_recording: str):
        self.path_to_recording = path_to_recording
        #self.data_array_path = self.path_to_recording/'DataArrays'
        self.output_data_array_path = self.path_to_recording/'output_data'
        self.raw_data_array_path = self.output_data_array_path/'raw_data'

    def load_skeleton_data(self):
        # freemocap_raw_data = np.load(self.data_array_path/'mediaPipeSkel_3d.npy')
        freemocap_raw_data = np.load(self.raw_data_array_path/'mediapipe3dData_numFrames_numTrackedPoints_spatialXYZ.npy')
        freemocap_raw_data = freemocap_raw_data[:,:,:]
        return freemocap_raw_data

    def save_skeleton_data(self, skeleton_data:np.ndarray, skeleton_file_name:str, settings_dict:dict):
        np.save(self.output_data_array_path/skeleton_file_name,skeleton_data)

        output_toml_name = self.output_data_array_path/'postprocessing_settings.toml'
        toml_string = toml.dumps(settings_dict)

        with open(output_toml_name, 'w') as toml_file:
            toml_file.write(toml_string)


class PostProcessingGUI(QWidget):
    def __init__(self,path_to_data_folder:Path):
        super().__init__()

        layout = QVBoxLayout()

        self.file_manager = FileManager(path_to_recording=path_to_data_folder)

        self.resize(1256, 1029)

        self.setWindowTitle("Freemocap Data Post-processing")

        self.tab_widget = QTabWidget()

        freemocap_raw_data = self.file_manager.load_skeleton_data()

        self.main_menu_tab = MainMenu(freemocap_raw_data=freemocap_raw_data)
        self.tab_widget.addTab(self.main_menu_tab, 'Main Menu')

        self.interp_tab = InterpolationMenu(freemocap_raw_data = freemocap_raw_data)
        self.tab_widget.addTab(self.interp_tab, 'Interpolation')
        # layout.addWidget(self.main_menu)

        self.filter_tab = FilteringMenu(freemocap_raw_data=freemocap_raw_data)
        self.tab_widget.addTab(self.filter_tab, 'Filtering')

        layout.addWidget(self.tab_widget)

        self.main_menu_tab.save_skeleton_data_signal.connect(self.file_manager.save_skeleton_data)

        self.setLayout(layout)


        f = 2

class MainWindow(QMainWindow):
    def __init__(self,path_to_data_folder:Path):
        super().__init__()

        layout = QVBoxLayout()

        widget = QWidget()
        postprocessing_window = PostProcessingGUI(path_to_data_folder)

        layout.addWidget(postprocessing_window)

        widget.setLayout(layout)
        self.setCentralWidget(widget)
        

def main(path_to_data_folder=None):
    if not path_to_data_folder:
        path_to_data_folder = Path(input("Enter path to data folder (no quotations around the path): "))
        

    app = QApplication([])
    win = MainWindow(path_to_data_folder)
    win.show()
    app.exec()



if __name__ == "__main__":

    main(Path(r"D:\2023-05-17_MDN_NIH_data\1.0_recordings\calib_3\sesh_2023-05-17_14_53_48_MDN_NIH_Trial3"))        





from PySide6.QtWidgets import QWidget,QVBoxLayout

import matplotlib
matplotlib.use('QtAgg')

from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

from skellyforge.freemocap_utils.postprocessing_widgets.visualization_widgets.mediapipe_skeleton_builder import mediapipe_indices

import numpy as np


class TimeSeriesPlotCanvas(FigureCanvasQTAgg):

    def __init__(self, parent=None, width=15, height=4, dpi=100):
        fig = Figure(figsize=(width, height * 0.7), dpi=dpi)
        self.x_ax = fig.add_subplot(311)
        self.y_ax = fig.add_subplot(312)
        self.z_ax = fig.add_subplot(313)

        fig.subplots_adjust(hspace=0.4)
        fig.tight_layout(pad=2.5)

        super(TimeSeriesPlotCanvas, self).__init__(fig)

class TimeSeriesPlotterWidget(QWidget):

    def __init__(self):
        super().__init__()

        self._layout = QVBoxLayout()
        self.setLayout(self._layout)

        self.lines = []


        self.fig, self.axes_list = self.initialize_skeleton_plot()

        toolbar = NavigationToolbar(self.fig, self)
        self._layout.addWidget(toolbar)
        self._layout.addWidget(self.fig)

    def initialize_skeleton_plot(self):
        fig = TimeSeriesPlotCanvas(self, width=12, height=9, dpi=100)
        self.x_ax = fig.figure.axes[0]
        self.y_ax = fig.figure.axes[1]
        self.z_ax = fig.figure.axes[2]

        self.axes_list = [self.x_ax,self.y_ax,self.z_ax]
        return fig, self.axes_list

    def get_mediapipe_indices(self,marker_to_plot):
        mediapipe_index = mediapipe_indices.index(marker_to_plot)
        return mediapipe_index
    

    def update_plot(self,marker_to_plot:str, original_freemocap_data:np.ndarray, processed_freemocap_data:np.ndarray, reset_axes = True):
        axes_names = ['X Axis (mm)', 'Y Axis (mm)', 'Z Axis (mm)']

        mediapipe_index = self.get_mediapipe_indices(marker_to_plot)

        for line in self.lines:
            line.remove()
        self.lines = []

        for dimension, (ax, ax_name) in enumerate(zip(self.axes_list, axes_names)):
            if reset_axes:
                ax.cla()

            line1, = ax.plot(original_freemocap_data[:, mediapipe_index, dimension], label='Original Data', alpha=.8, color='red')
            self.lines.append(line1)
            if processed_freemocap_data is not None:
                line2, = ax.plot(processed_freemocap_data[:, mediapipe_index, dimension], label='Processed Data', alpha=.6, color='blue')
                self.lines.append(line2)

            ax.set_ylabel(ax_name)

            if dimension == 2:  # put the xlabel only on the last plot
                ax.set_xlabel('Frame #')
            ax.legend()
        
        self.fig.figure.suptitle(f'{marker_to_plot} trajectory')

        self.fig.figure.canvas.draw_idle()



